# -*- coding: utf-8 -*-
"""
    trepan.io
    ~~~~~~~~~

    Trepan Input and Output routines

    :copyright: Copyright 2013, 2018
    :license: GPL3, see LICENSE for details.
"""

__docformat__ = 'restructuredtext'

from trepan.misc import pyfiles
__all__ = pyfiles(__file__)
