# This file is needs to be multi-lingual in both Python and POSIX
# shell which "exec()" or "source" it respectively.

# This file should define a variable VERSION which we use as the
# debugger version number.

VERSION="1.0.1"  # noqa
